package blusunrize.immersiveengineering.common.crafting;

import blusunrize.immersiveengineering.api.ApiUtils;
import blusunrize.immersiveengineering.api.crafting.IngredientStack;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;
import net.minecraft.world.World;
import net.minecraftforge.fluids.UniversalBucket;
import net.minecraftforge.oredict.ShapelessOreRecipe;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class RecipeShapelessIngredient extends ShapelessOreRecipe
{
	static IngredientStack[] tempIngredients;

	List<IngredientStack> ingredients;
	int nbtCopyTargetSlot = -1;
	int toolDamageSlot = -1;

	public RecipeShapelessIngredient(ItemStack result, Object... recipe)
	{
		super(result, saveIngredients(recipe));
		setIngredients(tempIngredients);
		tempIngredients = null;
	}

	public RecipeShapelessIngredient setIngredients(IngredientStack[] ingr)
	{
		ingredients = new ArrayList();
		input.clear();
		for(IngredientStack stack : ingr)
			if(stack != null)
			{
				ingredients.add(stack);
				input.add(stack.getShapedRecipeInput());
			}
		return this;
	}

	public List<IngredientStack> getIngredients()
	{
		return ingredients;
	}

	public RecipeShapelessIngredient setNBTCopyTargetRecipe(int slot)
	{
		this.nbtCopyTargetSlot = slot;
		return this;
	}

	public RecipeShapelessIngredient setToolDamageRecipe(int slot)
	{
		this.toolDamageSlot = slot;
		return this;
	}

	public static Object[] saveIngredients(Object... recipe)
	{
		Object[] converted = new Object[recipe.length];
		tempIngredients = new IngredientStack[recipe.length];
		for(int i = 0; i < recipe.length; i++)
		{
			IngredientStack ingr = ApiUtils.createIngredientStack(recipe[i]);
			if(ingr != null)
			{
				tempIngredients[i] = ingr;
				converted[i] = Blocks.field_150480_ab;
			}
		}
		return converted;
	}

	@Override
	public ItemStack func_77572_b(InventoryCrafting matrix)
	{
		if(nbtCopyTargetSlot >= 0 && nbtCopyTargetSlot < ingredients.size())
			for(int i = 0; i < matrix.func_70302_i_(); i++)
			{
				ItemStack slot = matrix.func_70301_a(i);
				if(ingredients.get(nbtCopyTargetSlot).matchesItemStack(slot))
				{
					ItemStack out = output.func_77946_l();
					if(!matrix.func_70301_a(nbtCopyTargetSlot).func_190926_b() && matrix.func_70301_a(nbtCopyTargetSlot).func_77942_o())
						out.func_77982_d(matrix.func_70301_a(nbtCopyTargetSlot).func_77978_p().func_74737_b());
					return out;
				}
			}
		return super.func_77572_b(matrix);
	}

	@Override
	public NonNullList<ItemStack> func_179532_b(InventoryCrafting inv)
	{
		NonNullList<ItemStack>  remains = super.func_179532_b(inv);
		for(int i = 0; i < remains.size(); i++)
		{
			ItemStack s = inv.func_70301_a(i);
			ItemStack remain = remains.get(i);
			if(toolDamageSlot >= 0 && toolDamageSlot < ingredients.size())
			{
				ItemStack tool = ItemStack.field_190927_a;
				if(remain.func_190926_b() && !s.func_190926_b() && ingredients.get(toolDamageSlot).matchesItemStack(s))
					tool = s.func_77946_l();
				else if(!remain.func_190926_b() && ingredients.get(toolDamageSlot).matchesItemStack(remain))
					tool = remain;
				if(!tool.func_190926_b() && tool.func_77973_b().func_77645_m())
				{
					tool.func_77964_b(tool.func_77952_i() + 1);
					if(tool.func_77952_i() > tool.func_77958_k())
						tool = ItemStack.field_190927_a;
					remains.set(i, tool);
				}
			}
			if(!s.func_190926_b() && remain.func_190926_b() && s.func_77973_b() instanceof UniversalBucket)
			{
				ItemStack empty = ((UniversalBucket) s.func_77973_b()).getEmpty();
				if(!empty.func_190926_b())
					remains.set(i, empty.func_77946_l());
			}
		}
		return remains;
	}

	@Override
	public boolean func_77569_a(InventoryCrafting matrix, World world)
	{
		ArrayList<IngredientStack> required = new ArrayList(ingredients);

		for(int i = 0; i < matrix.func_70302_i_(); i++)
		{
			ItemStack slot = matrix.func_70301_a(i);
			if(!slot.func_190926_b())
			{
				boolean inRecipe = false;
				Iterator<IngredientStack> iterator = required.iterator();
				while(iterator.hasNext())
				{
					IngredientStack next = iterator.next();
					if(next.matchesItemStack(slot))
					{
						inRecipe = true;
						iterator.remove();
						break;
					}
				}
				if(!inRecipe)
					return false;
			}
		}
		return required.isEmpty();
	}
}
